%==========================================================================
%
%   Numerical Derivative Calculation Using the Central Difference Method
%
%   Df = central_difference(f,deltaX) calculates the numerical derivative
%   of the function samples in 'f' by using the central-difference method.
%   The scalar 'deltaX' is the spacing between grid samples, which are
%   assumed to be uniform.
%
%   Ordinarily, this should lead to an output vector that is one sample
%   less than the original input vector.  To avoid this confusion, the
%   derivatives are all assumed to be calculated at the original function
%   samples in 'f'.  That means each central difference calculation is
%   performed using the two neighboring points.  For example, see the
%   geometry below:
%
%       |
%       |
%       |                   f(x2)
%       |                   *               
%       |       f(x1)
%       |       *                       f(x3)
%       |                               *
%       |
%       ------------------------------------------
%               x1          x2          x3
%
%
%       The derivative at x2 is calculated using:
%
%                           f(x3) - f(x1)
%               f'(x2) = --------------------
%                              2*DeltaX
%
%   Notice that this makes the derivative calculation a little less
%   accurate than it could be since the spacing is doubled.  However, this
%   also removes the ambiguity of "where" exactly the derivative is
%   located.  Otherwise, the derivatives must be evaluated on a staggered
%   grid between the original samples, which requires more book-keeping.
%
%   Also take note that the first and last elements in F cannot be
%   evaluated using a central difference.  So to maintain the same theme,
%   the first and last samples are simply evaluated using a forward and
%   backward finite difference.
%
%                           f(2) - f(1)
%               f'(1) = --------------------
%                              DeltaX
%
%
%                           f(N) - f(N-1)
%               f'(N) = --------------------
%                              DeltaX
%
%
%     INPUTS
%           f:  Vector of function samples.  
%      deltaX:  Incremental spacing of the x-axis.  If not specified, then
%                 deltaX = 1.  
%
%     OUTPUT
%          Df:  Numerical derivative of the vector f.
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright June 16, 2013

function [Df] = central_difference(f,dx)

%  Assume unit increments if not specified.
if (nargin == 1)
  dx = 1;
end

%  Number of samples in the vector.
N = length(f);

%  Initialize slope vector.
Df = zeros(1,N);

%  Forward finite difference on the first point.
Df(1) = (f(2)-f(1))/dx;

%  Calculate the central differences on the inner points.
Df(2:N-1) = (f(3:N) - f(1:N-2))/(2*dx);

%  Backward finite difference on the last point.
Df(N) = (f(N) - f(N-1))/dx;

return;
