%==========================================================================
%
%                           SYSTEM GEOMETRY:
%
%             z = D(1)        z = D(2) ...    z = D(M-1)
%               |               |               |
%               |               |               |
%               |               |               |
%      A(1)     |     A(2)  	|               |   A(M)
%    ------>    |    ------>    |      ...      |  ------>      ^ +x
%               |               |               |               |
%               |               |               |               |
%    <------    |    <------    |      ...      |  <------      |    +z
%      B(1)     |     B(2)  	|               |   B(M)        ------>
%               |               |               |
%               |               |               |
%               |               |               |
%               |               |               |
%      n(1)     |     n(2)      |      ...      |   n(M)
%               |               |               |
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright June 16, 2013

function [R,T,AF,Sz,L] = laser_power_abs(L0,D,e_r,m_r,THi,TE,x,z)

%  Solve for field amplitudes and wavenumbers.
[A,B,kx,kz] = solve_layered_stack(e_r,m_r,D,L0,THi,TE);

%  Calculate reflectance.
R = abs(B(1))^2 / abs(A(1))^2 ;

%  Stop here if R is the only parameter needed.
if nargout == 1
    return;
end

%==========================================================================
%  Calculate transmittance.
%==========================================================================

%  Total number of layers.
M = length(e_r);

%  Calculate normal component of propagation constants.
beta_M = real(kz(M));       %  Target region.
beta_1 = real(kz(1));       %  Incident region.

%  Transmission coefficient.
tau = A(M) / A(1) ;

%  Calculate loss parameters.  These look more complicated than they really
%  are.  For lossless materials, they simplify to the ratio of relative
%  permeabilities (TE) or relative permittivities (TM).  However, when
%  there is loss in the system, these parameters become complex and need to
%  be handled more carefully.  Just remember that S = 0.5*real(E x conj(H))
%  is always the ultimate definition of Poynting power density. 
if TE
    U_M = real(1/conj(m_r(M)));
    U_1 = real(1/conj(m_r(1)));
else
    U_M = real(1/conj(e_r(M)));
    U_1 = real(1/conj(e_r(1)));
end

%  Calculate transmittance.
T = abs(tau)^2 * beta_M/beta_1 * U_M / U_1;

%  Stop here if R and T are the only parameters needed.
if nargout == 2
    return;
end

%==========================================================================
%  Calculate total absorption factor.
%==========================================================================
AF = 1 - R - T;

%  Stop here if R, T, and A are the only parameters needed.
if nargout == 3
    return;
end

%==========================================================================
%  Prepare to calculate power absorption per unit volume.
%==========================================================================

%  Physical constants.
m_0 = 4*pi*1e-7;        %  Permeability of free-space (H/m).
e_0 = 8.854e-12;        %  Permittivity of free-space (F/m).
c_0 = 1/sqrt(m_0*e_0);  %  Speed of light in free-space (m/s).

%  Excitation parameters.
k0 = 2*pi/L0;           %  Free-space wavenumber (rad/m).
w  = k0*c_0;            %  Angular frequency of excitation (rad/s).

%  number of samples.
Nx = length(x);
Nz = length(z);

%  Generate position matrices.
[X, Z] = meshgrid(x,z);

%==========================================================================
%   Solve for total field intensity and power density.
%==========================================================================

%  Instantiate primary and secondary field components.  By definition, the
%  "primary" field component is the transverse component.  From the system
%  geometry above, this is always the y-component.  So if TE == 1, then Vy
%  becomes Ey, while Vx and Vz become Hx and Hz.  Likewise, for TE == 0,
%  then Vy is defined as Hy while Vx and Vz become Ex and Ez.  
Vy = zeros(Nz,Nx);  %  Primary field component. 
Vx = zeros(Nz,Nx);  %  Secondary x.
Vz = zeros(Nz,Nx);  %  Secondary y.

%  
L = zeros(Nz,Nx);  %  Secondary y.

%  Start generating fields for each region.  Use an underscore to denote
%  the "temp" field variable for a region that gets added to the whole.
for m = 1:M
    
    %  In Region 1, the incident wave is referenced against z1 = 0.
    %  Otherwise, it is referenced against the previous distance.
    if m == 1
        z1 = D(1);
    else
        z1 = D(m-1);
    end
    
    %  In region M, there should be no reverse-propagating wave.  The value
    %  therefore should not matter for z2, and we can just pick anything.
    %  For the forward-propagating wave, however, we use the final value in
    %  D.
    if m == M
        z2 = D(m-1);    %  This value does not matter when B(M) = 0.
    else
        z2 = D(m);
    end
    
    %  Generate the forward and reverse propagating primary fields.
    Vy1 = A(m)*exp(+1j*(kx*X + kz(m)*(Z-z1)) );  %  Forward.
    Vy2 = B(m)*exp(+1j*(kx*X - kz(m)*(Z-z2)) );  %  Reverse.
    
    %  Add primary fields together.  Rmember that the underscore is being
    %  used to emphasize the temporary variable.
    V_y = Vy1 + Vy2;
    
    %  Forward and reverse secondary fields (normalized).
    Vx1 = A(m)*exp(+1j*(kx*X + kz(m)*(Z-z1)) );     %  Forward x.
    Vx2 = B(m)*exp(+1j*(kx*X - kz(m)*(Z-z2)) );     %  Reverse x.
    Vz1 = A(m)*exp(+1j*(kx*X + kz(m)*(Z-z1)) );     %  Forward z.
    Vz2 = B(m)*exp(+1j*(kx*X - kz(m)*(Z-z2)) );     %  Reverse z.
    
    %  Scale the secondary fields and add together.
    if TE
        V_x = ( kz(m) / (w*m_0*m_r(m)) ) * (-Vx1 + Vx2);
        V_z = ( kx    / (w*m_0*m_r(m)) ) * ( Vz1 + Vz2);
    else
        V_x = (kz(m) / (w*e_0*e_r(m)) ) * ( Vx1 - Vx2);
        V_z = (kx    / (w*e_0*e_r(m)) ) * (-Vz1 - Vz2);
    end

    %  The next step is to zero out all the values beyond the region of
    %  interest.  This is kind of inefficient because I'm calculating the
    %  fields everywhere outside of the region of interest.  Consider
    %  fixing this later if things are too slow.
    if m ~= 1
        V_y( Z < D(m-1) ) = 0;
        V_x( Z < D(m-1) ) = 0;
        V_z( Z < D(m-1) ) = 0;
    end
    
    if m ~= M
        V_y( (Z >= D(m) ) ) = 0;
        V_x( (Z >= D(m) ) ) = 0;
        V_z( (Z >= D(m) ) ) = 0;
    end
    
    %  Add fields to the total system.  Again, the underscored variable is
    %  the "temp" for this layer.
    Vy = Vy + V_y;
    Vx = Vx + V_x;
    Vz = Vz + V_z;
         
    %  Calculate power absorption per unit volume.  Note that I am not
    %  accounting for lossy magnetic materials, so don't trust this output
    %  if m_r is complex.  
    if TE
        L = L + 0.5*w*abs(V_y).^2 .* imag(e_0*e_r(m));
    else
        L = L + 0.5*w*( abs(V_x).^2 + abs(V_z).^2 ) .* imag(e_0*e_r(m));
    end
    
end

%  Calculate the time-averaged Poynting vector.
if TE
    Sz = -0.5*real(Vy.*conj(Vx));
else
    Sz =  0.5*real(Vx.*conj(Vy));
end

